<?php
//Block direct access to the file
if ( !function_exists( 'add_action' ) ) {
	exit; 
}

/**
 * View currently vulnerable plugins and manage the Plugin Vulnerabilities plugin's settings.
 */
class pv_wp_cli extends WP_CLI_Command {
	/**
	 * Activate API license key.
	 *
	 * ## OPTIONS
	 * --license-key=<key>
	 * : API license key
	 *
	 * --license-email=<email>
	 * : API license email
	 *
	 * ## EXAMPLES
	 *
	 * wp plugin-vulnerabilities activate --license-key=key --license-email=email
	 *
	 */
	function activate( $args, $assoc_args ) {
		$instance = trim( wp_generate_password( 12, false ) );
		$response = wp_remote_post( 'https://www.pluginvulnerabilities.com/?wc-api=am-software-api', array(
			'method' => 'POST',
			'timeout' => 45,
			'redirection' => 5,
			'httpversion' => '1.0',
			'blocking' => true,
			'headers' => array(),
			'body' => array( 'email' => $assoc_args['license-email'], 'licence_key' => $assoc_args['license-key'], 'request' => 'activation', 'product_id' => 'Subscription', 'instance' => $instance, 'platform' => get_site_url() ),
			'cookies' => array()
			)
		);
		if ( is_wp_error( $response ) )
			$activation_error = $response->get_error_message();
		else {
			$activation_response = json_decode($response['body'], true);
			if ( isset($activation_response['activated']) && $activation_response['activated'] == 'active' ) {
				update_option( 'plugin_vulnerabilities_api_license_key', $assoc_args['license-key'] );
				update_option( 'plugin_vulnerabilities_api_license_email', $assoc_args['license-email'] );
				update_option( 'plugin_vulnerabilities_api_license_instance', $instance );
				wp_clear_scheduled_hook( 'plugin_vulnerabilities_cron' );
				if ( get_option( 'plugin_vulnerabilities_check_frequency' ) && ( get_option( 'plugin_vulnerabilities_check_frequency' )=='hourly' || get_option( 'plugin_vulnerabilities_check_frequency' )=='twicedaily' || get_option( 'plugin_vulnerabilities_check_frequency' )=='daily' ) )
					wp_schedule_event( time(), get_option( 'plugin_vulnerabilities_check_frequency' ), 'plugin_vulnerabilities_cron' );
				else 
					wp_schedule_event( time(), 'twicedaily', 'plugin_vulnerabilities_cron' );
			}
			else if ( isset($activation_response['error']) ) {
				$activation_error = $activation_response['error'];
			}
			else
				$activation_error = 'Unknown Error';
		}
		if ( isset ($activation_error ) ) {
			WP_CLI::line( 'Could not activate API license key.' );
			WP_CLI::line( 'Error Given: '.$activation_error );
		}
		else{
			WP_CLI::line( 'API license key activated.' );
		}
	}
     
     
	/**
	 * Display vulnerable plugins based on last cron check.
	 *
	 * ## OPTIONS
	 * [--format=<format>]
	 * : Render output in a particular format.
	 * ---
	 * default: table
	 * options:
	 *  - table
	 *  - csv
	 *  - json
	 *  - count
	 *  - yaml
	 * ---
	 */
	function vulnerable( $args, $assoc_args ) {
		if ( get_option( 'plugin_vulnerabilities_api_license_key' )  ){
			if ( get_option( 'plugin_vulnerabilities_vulnerability_data_cache' ) ) {
				$vulnerability_cache = get_option( 'plugin_vulnerabilities_vulnerability_data_cache' );
				if ($vulnerability_cache == 'None' )
					WP_CLI::line( 'The installed versions of plugins are not currently known to be vulnerable.' );
				else {
					$installed_plugins = plugin_vulnerabilities_gather_plugins();
					$plugin_vulnerabilities = array();
					foreach ($vulnerability_cache as $vulnerability) {
						if ( array_key_exists ($vulnerability['Path'],$installed_plugins) ) {
							if ( version_compare( $installed_plugins[$vulnerability['Path']]['Version'], $vulnerability['First Version'], '>=') && version_compare( $installed_plugins[$vulnerability['Path']]['Version'], $vulnerability['Last Version'], '<=') ) {
								array_push($plugin_vulnerabilities, array(
									'Name' => $installed_plugins[$vulnerability['Path']]['Name'],
									'Slug' => $vulnerability['Path'],
									'First Version' => $vulnerability['First Version'],
									'Last Version' => $vulnerability['Last Version'],
									'Likelihood of Exploitation' => ucwords( $vulnerability['Likelihood Of Exploitation'] ),
									'Type of Vulnerability' => ucwords( $vulnerability['Type Of Vulnerability'] ),
									'Details URL' => $vulnerability['URL']
									)
								);
							}
						}
					}
					if ( !empty ( $plugin_vulnerabilities ) ) {
						WP_CLI\Utils\format_items( $format, $plugin_vulnerabilities, array( 'Name', 'Slug', 'First Version', 'Last Version', 'Likelihood of Exploitation', 'Type of Vulnerability', 'Details URL' ) );
					}
					else
						WP_CLI::line( 'The installed versions of plugins are not currently known to be vulnerable.' );
				}				
			}
			else
				WP_CLI::line( 'The installed versions of plugins are not currently known to be vulnerable.' );
		}
		else
			WP_CLI::line( 'An API license key has not been set up.' );
	}

  /**
   * Display and change the plugin's settings.
	 *
	 * ## OPTIONS
	 * [--frequency=<frequency>]
	 * : Frequency of vulnerablity checks: hourly, twicedaily, daily
	 * ---
	 * options:
	 *  - hourly
	 *  - twicedaily
	 *  - daily
	 * ---
	 *
	 * [--automaticupdateslikelihood=<likelihood>]
	 * : Automatically update a plugin if a vulnerability with a likelihood of exploitation of at least is fixed: none, low, medium, high
	 * ---
	 * options:
	 *  - none
	 *  - low
	 *  - medium
	 *  - high
	 * ---
	 *
	 * [--email=<email>]
	 * : Change email address for alerts. With the value set to disabled, email alerts will be disabled, and set to remove will remove a custom email address.
	 *
	 */
	function settings( $args, $assoc_args ) {
		if ( get_option( 'plugin_vulnerabilities_api_license_key' ) ) {
			if ( isset ( $assoc_args['frequency'] ) ) {
				if ( ( get_option( 'plugin_vulnerabilities_check_frequency' ) && ( get_option( 'plugin_vulnerabilities_check_frequency' )!= $assoc_args['frequency'] ) ) || ( !get_option( 'plugin_vulnerabilities_check_frequency' ) && $assoc_args['frequency'] != 'twicedaily' ) ) {
					update_option( 'plugin_vulnerabilities_check_frequency', $assoc_args['frequency'] );
					wp_clear_scheduled_hook( 'plugin_vulnerabilities_cron' );
					wp_schedule_event( time(), get_option( 'plugin_vulnerabilities_check_frequency' ), 'plugin_vulnerabilities_cron' );
				}
			}
			if ( isset ( $assoc_args['automaticupdateslikelihood'] ) ) {
				if ( in_array ( $assoc_args['automaticupdateslikelihood'], array('none','low','medium','high') ) ) {
					update_option( 'plugin_vulnerabilities_automatic_updates', $assoc_args['automaticupdateslikelihood'] );
				}
			}
			if ( isset ($assoc_args['email'] ) ) {
				if ( $assoc_args['email'] == 'remove') {
					delete_option( 'plugin_vulnerabilities_email_address') ;
				}
				if ( $assoc_args['email'] == 'disabled') {
					update_option( 'plugin_vulnerabilities_email_address', 'disabled' );
				}
				else if ( is_email ($assoc_args['email']) ) {
					update_option( 'plugin_vulnerabilities_email_address', $assoc_args['email'] );
				}
				else {
					WP_CLI::line( 'The new settings for the email address, '.$assoc_args['email'].', doesn\'t appear to be valid.' );
					WP_CLI::line( '' );
				}
			}
			WP_CLI::line( 'API License key: '.get_option( 'plugin_vulnerabilities_api_license_key' ) );
			WP_CLI::line( 'API License Email: '.get_option( 'plugin_vulnerabilities_api_license_email' ) );
			if ( get_option( 'plugin_vulnerabilities_check_frequency' ) ) {
				switch ( get_option( 'plugin_vulnerabilities_check_frequency' ) ) {
					case 'hourly':
						$frequency =  'Hour';
						break;
					case 'twicedaily':
						$frequency =  '12 Hours';
						break;
					case 'daily':
						$frequency =  'Day';
						break;
				}
			}
			else
				$frequency =  '12 Hours';
			WP_CLI::line( 'Check For Vulnerabilities Every: '.$frequency );
			if ( get_option( 'plugin_vulnerabilities_automatic_updates' ) ) {
				switch ( get_option( 'plugin_vulnerabilities_automatic_updates' ) ) {
					case 'none':
						$frequency =  'Never';
						break;
					case 'low':
						$frequency =  'Low';
						break;
					case 'medium':
						$frequency =  'Medium';
						break;
					case 'high':
						$frequency =  'High';
						break;
				}
			}
			else
				$frequency =  'Never';
			WP_CLI::line( 'Automatically Update a Plugin if a Vulnerability With a Likelihood of Exploitation of at Least is Fixed: '.$frequency );
			if ( get_option( 'plugin_vulnerabilities_email_address') ) { 
				if ( get_option( 'plugin_vulnerabilities_email_address') == 'disabled' )
					WP_CLI::line( 'Email alerts disabled.' );
				else
					WP_CLI::line( 'Email Address for Alerts: '.get_option( 'plugin_vulnerabilities_email_address') );
				
			}
			else
				WP_CLI::line( 'Email Address for Alerts: '.get_option( 'admin_email' ) );
		
		}
		else
			WP_CLI::line( 'An API license key has not been set up.' );
	}
	
	/**
     * Deactivate API license key.
     */
	function deactivate() {
		if ( get_option( 'plugin_vulnerabilities_api_license_key') ) {
			wp_remote_post( 'https://www.pluginvulnerabilities.com/?wc-api=am-software-api', array(
				'method' => 'POST',
				'timeout' => 45,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking' => true,
				'headers' => array(),
				'body' => array( 'email' => get_option( 'plugin_vulnerabilities_api_license_email'), 'licence_key' => get_option( 'plugin_vulnerabilities_api_license_key'), 'request' => 'deactivation', 'product_id' => 'Subscription', 'instance' => get_option( 'plugin_vulnerabilities_api_license_instance'), 'platform' => get_site_url() ),
				'cookies' => array()
				)
			);
			delete_option( 'plugin_vulnerabilities_api_license_key');
			delete_option( 'plugin_vulnerabilities_api_license_email');
			delete_option( 'plugin_vulnerabilities_api_license_instance');
			if (!get_option( 'plugin_vulnerabilities_check_frequency' ) || ( get_option( 'plugin_vulnerabilities_check_frequency' )!='hourly') ) {
				wp_clear_scheduled_hook( 'plugin_vulnerabilities_cron' );
				wp_schedule_event( time(), 'hourly', 'plugin_vulnerabilities_cron' );
			}
			WP_CLI::line( 'The API license key has been deactivated.' );
		}
		else
			WP_CLI::line( 'An API license key has not been set up.' );
	}
}

WP_CLI::add_command( 'plugin-vulnerabilities', 'pv_wp_cli' );